<?php
// +-----------------------------------------------------------------------------+
// | evList A calendar solution for Geeklog                                      |
// | Date: Aug, 2007                                                             |
// +-----------------------------------------------------------------------------+
// | functions.inc - Geeklog Plugin API functions and support functions          |
// +-----------------------------------------------------------------------------+
// | Copyright (C) 2007 by:                                                      |
// | evlist Plugin Author:                                                       |
// | Alford Deeley - ajdeeley@summitpages.ca                                     |
// +-----------------------------------------------------------------------------+
// | Geeklog Common Code                                                         |
// | Copyright (C) 2000-2007 by the following authors:                           |
// | Authors: Tony Bibbs        - tony@tonybibbs.com                             |
// |          Mark Limburg      - mlimburg@users.sourceforge.net                 |
// |          Jason Whittenburg - jwhitten@securitygeeks.com                     |
// |          Dirk Haun         - dirk@haun-online.de                            |
// +-----------------------------------------------------------------------------+
// |                                                                             |
// | This program is licensed under the terms of the GNU General Public License  |
// | as published by the Free Software Foundation; either version 2              |
// | of the License, or (at your option) any later version.                      |
// |                                                                             |
// | This program is distributed in the hope that it will be useful,             |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of              |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                        |
// | See the GNU General Public License for more details.                        |
// |                                                                             |
// | You should have received a copy of the GNU General Public License           |
// | along with this program; if not, write to the Free Software Foundation,     |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             |
// |                                                                             |
// +-----------------------------------------------------------------------------+

//If you change $CONF_EVLIST['pi_name'] in evlist's config.php file then
//the following path needs to be edited because the path follows the name.
require_once ($_CONF['path'] . 'plugins/evlist/config.php');

$langfile = $_CONF['path'] . 'plugins/evlist/language/' . $_CONF['language'] . '.php';
if (file_exists ($langfile)) {
    require_once ($langfile);
}
else {
    require_once ($_CONF['path'] . 'plugins/evlist/language/english.php');
}

function plugin_chkVersion_evlist() {
    global $CONF_EVLIST; 
    return $CONF_EVLIST['version'];
}

/**
* Removes the data structures for this plugin from the Geeklog database
*
* This may get called by the install routine to undo anything done to this
* point.  To do that, $steps will have a list of steps to undo
*
* @steps   Array    Holds all the steps that have been completed by the install
*
*/
function plugin_uninstall_evlist()
{
    global $_TABLES;
    $FEATURES = array ('evlist.edit');
    $TABLES = array ('evlist_settings','evlist_events','evlist_categories',
        'evlist_lookup','evlist_remlookup','evlist_dateformat',
        'evlist_timeformat','evlist_submissions');

    // Unregister the plugin with Geeklog
    COM_errorLog('Attempting to unregister the evList plugin from Geeklog',1);
    DB_query("DELETE FROM {$_TABLES['plugins']} WHERE pi_name = 'evlist'",1);
    
    // Drop Plugun tables
    foreach($TABLES as $table) {
        $t = $_TABLES["$table"];
        COM_errorLog("Removing Table $t",1);
        DB_query("DROP TABLE $t",1);
    }

    // Remove security for this plugin

    $grp_id = DB_getItem($_TABLES['groups'], 'grp_id', "grp_name = 'evList Admin'");
    
    // Remove the evlist admin group
    COM_errorLog('Attempting to remove the evList Admin Group', 1);
    DB_query("DELETE FROM {$_TABLES['groups']} WHERE grp_id = $grp_id");
    
    // Remove root users from the group
    COM_errorLog('Attempting to remove root users from admin of evList');
    DB_query("DELETE FROM {$_TABLES['group_assignments']} WHERE ug_main_grp_id = $grp_id");

    // Remove all the associated features - access rights.
    foreach ($FEATURES as $feature) {
        COM_errorLog("Removing $feature feature and rights to it",1);
        $feat_id = DB_getItem($_TABLES['features'], 'ft_id', "ft_name = '$feature'");
        DB_query("DELETE FROM {$_TABLES['access']} WHERE acc_ft_id = $feat_id",1);
        DB_query("DELETE FROM {$_TABLES['features']} WHERE ft_id = $feat_id",1);
    }
    
    // Remove the plugin specific blocks that were added
    DB_query("DELETE FROM {$_TABLES['blocks']} WHERE phpblockfn = 'phpblock_evlist_upcoming'",1);
    
    // Remove speed limit entries if any exist
    DB_query("DELETE FROM {$_TABLES['speedlimit']} WHERE type = 'eventrem' OR type = 'event'",1);

    COM_errorLog('...success',1);
    COM_errorLog('leaving plugin_uninstall_evlist',1);

    return true;
}

function plugin_upgrade_evlist(){
    global $_CONF, $_TABLES, $CONF_EVLIST;
    
    $pi_name = 'evlist';
    $version = DB_getItem($_TABLES['plugins'], 'pi_version', "pi_name = '$pi_name'");
    
    if ($version < '1.0.2') {
        //upgrade from initial release
        DB_query("ALTER TABLE {$_TABLES['evlist_settings']}
            ADD limit_list TINYINT(2) NOT NULL DEFAULT '5',
            ADD limit_block TINYINT(2) NOT NULL DEFAULT '5',
            ADD limit_summary TINYINT(3) UNSIGNED NOT NULL DEFAULT '128'
            ");
        DB_query("UPDATE {$_TABLES['plugins']} SET pi_version = '{$CONF_EVLIST['version']}' WHERE pi_name = '$pi_name'");
    }
    
    if ($version < '1.1') {
        //upgrade to version 1.1
        DB_query("ALTER TABLE {$_TABLES['evlist_settings']}
            ADD enable_reminders TINYINT(1) NOT NULL DEFAULT '1',
            ADD event_passing TINYINT(1) NOT NULL default '2'
            ");
        DB_query("ALTER TABLE {$_TABLES['evlist_events']}
            ADD enable_reminders TINYINT(1) NOT NULL default '1' AFTER hits,
            CHANGE time_start1 time_start1 VARCHAR(8) NOT NULL DEFAULT '0'
            ");
        DB_query("ALTER TABLE {$_TABLES['evlist_submissions']}
            ADD enable_reminders TINYINT(1) NOT NULL default '0' AFTER hits,
            CHANGE time_start1 time_start1 VARCHAR(8) NOT NULL DEFAULT '0'
            ");
        if (file_exists ($_CONF['path'] . 'plugins/evlist/sql/def_events.php')) {
            //because the default events have been updated with new information
            include_once ($_CONF['path'] . 'plugins/evlist/sql/def_events.php');
            DB_query($DEFVALUES[1]);
        } else {
            //you won't get the new info, but your default events will still work.
            DB_query("UPDATE IGNORE {$_TABLES['evlist_events']}
                SET time_start1 = '0' WHERE id = '20070924175337252'
                OR id = '20070922110402423' OR id = '20070924133400211'
                ");
        }
        DB_query("CREATE TABLE {$_TABLES['evlist_remlookup']} (
            id VARCHAR(40) NOT NULL,
            eid VARCHAR(40) NOT NULL,
            date_start INT UNSIGNED NOT NULL,
            timestamp INT UNSIGNED,
            email VARCHAR(96) NOT NULL,
            days_notice SMALLINT(3) NOT NULL default '7',
            UNIQUE eid (eid,timestamp,email,days_notice)
            ) TYPE=MyISAM
            ");
        DB_query("UPDATE {$_TABLES['plugins']} SET pi_version = '{$CONF_EVLIST['version']}' WHERE pi_name = '$pi_name'");
    }
    
    return true;
}

function plugin_cclabel_evlist() {
    global $_CONF,$LANG_EVLIST00;
    if (SEC_hasRights('evlist.edit')) {
        return array($LANG_EVLIST00[1],$_CONF['site_admin_url'] . "/plugins/evlist/index.php",$_CONF['site_url'] . '/evlist/images/evlist.png');
    }
}

function plugin_getadminoption_evlist() {
    global $_TABLES,$_CONF,$LANG_EVLIST00;

    if (SEC_hasRights('evlist.edit')) {
        $numevents = DB_getITEM($_TABLES['evlist_events'],"count(*)");
        return array($LANG_EVLIST00[1], $_CONF['site_admin_url'] . '/plugins/evlist/index.php', $numevents);
    }
}

function plugin_getuseroption_evlist() {
    global $_CONF, $_TABLES, $LANG_EVLIST00;

    $useroption = DB_getItem($_TABLES['evlist_settings'], 'usermenu_option');
    if ($useroption == '2') {
        return array($LANG_EVLIST00[2], COM_buildURL($_CONF['site_url'] . '/evlist/index.php?op=list'), 0);
    }
    if ($useroption == '1' && SEC_hasRights('evlist.edit')) {//this is default
        return array($LANG_EVLIST00[3], $_CONF['site_url'] . '/evlist/event.php?op=new', 0);
    }
}

function plugin_getmenuitems_evlist() {
    global $_CONF, $_TABLES, $LANG_EVLIST00;
    $menuitems = array();
    if (DB_getItem($_TABLES['evlist_settings'], 'enable_menuitem') == '1') {
        $menuitems[$LANG_EVLIST00[6]] = $_CONF['site_url'] . '/evlist/index.php?op=list';
    }
    //I might add a couple more options for menuitems also to be configured from the admin panel.
    return $menuitems;
}

function plugin_showstats_evlist($showsitestats)
{
    global $_CONF, $_TABLES, $LANG_EVLIST00;
    $retval = '';

    $result = DB_query ("SELECT id, title, hits FROM {$_TABLES['evlist_events']} WHERE hits > 0" . COM_getPermSQL('AND') . ' ORDER BY hits DESC LIMIT 10');
    $nrows  = DB_numRows ($result);
    if ($nrows > 0) {
        require_once( $_CONF['path_system'] . 'lib-admin.php' );
        $header_arr = array(
            array('text' => $LANG_EVLIST00[13], 'field' => 'eid', 'header_class' => 'stats-header-title'),
            array('text' => $LANG_EVLIST00[11], 'field' => 'hits', 'header_class' => 'stats-header-count', 'field_class' => 'stats-list-count'),
        );
        $data_arr = array();
        $text_arr = array('has_menu'     => false,
                          'title'        => $LANG_EVLIST00[14],
        );
        for ($i = 0; $i < $nrows; $i++) {
            $A = DB_fetchArray ($result);
            $A['title'] = COM_stripslashes ($A['title']);
            $A['eid'] = "<a href=\"" . COM_buildUrl ($_CONF['site_url']
                      . "/evlist/event.php?eid={$A['id']}"). "\">{$A['title']}</a>";
            $A['hits'] = COM_NumberFormat ($A['hits']);
            $data_arr[$i] = $A;
        }
        $retval .= ADMIN_simpleList("", $header_arr, $text_arr, $data_arr);
    } else {
        $retval .= COM_startBlock ($LANG_EVLIST00[14]);
        $retval .= $LANG_EVLIST00[15];
        $retval .= COM_endBlock();
    }

    return $retval;
}

function plugin_statssummary_evlist() {
    global $_TABLES, $LANG_EVLIST00;
    
    $result = DB_query("SELECT COUNT(*) AS numrows FROM {$_TABLES['evlist_events']} WHERE status='1'" . COM_getPermSQL('AND'));
    list ($numrows) = DB_fetchArray($result);
    
    return array($LANG_EVLIST00[12], COM_numberFormat($numrows));
}

function plugin_searchtypes_evlist() {
    global $LANG_EVLIST00;

    $type_label['evlist'] = $LANG_EVLIST00[12];
    return $type_label;
}

function plugin_dopluginsearch_evlist($query, $datestart, $dateend, $topic, $type, $author) {
    global $_CONF, $_TABLES, $LANG_EVLIST00;
    
    if (DB_getItem($_TABLES['evlist_settings'], 'allow_anon_view') != '1') {
        //return noresults
        $plugin_results = new Plugin();
        $plugin_results->plugin_name = $LANG_EVLIST00[1];
        $plugin_results->searchlabel = $LANG_EVLIST00[7] . $LANG_DQ[8];
        return $plugin_results;
    } else {
        if (empty($type)) {
            $type = 'all';
        }

        // Bail if we aren't supppose to do our search
        if ($type <> 'all' AND $type <> 'evlist') {
            $plugin_results = new Plugin();
            $plugin_results->plugin_name = $LANG_EVLIST00[1];
            $plugin_results->searchlabel = $LANG_EVLIST00[7] . $LANG_DQ[8];
            return $plugin_results;
        }
        
        $query = addslashes($query);
        
        $sql = "SELECT DISTINCT ID, title, date_start1, hits, e.owner_id, username";
        $sql .= " FROM {$_TABLES['evlist_events']} e, {$_TABLES['users']} gl";
        $sql .= " WHERE (e.Status='1' AND e.owner_id=gl.uid)";
        $sql .= " AND (title like '%$query%'";
        $sql .= " OR summary like '%$query%'";
        $sql .= " OR full_description like '%$query%'";
        $sql .= " OR location like '%$query%'";
        $sql .= " OR contact like '%$query%')";
        if (!empty($author)) {
            $sql .= " AND e.owner_id='$author'";
        }
        $sql .= COM_getPermSQL('AND');
        $sql .= " ORDER BY date_start1 DESC";
        
        // Perform search
        $result = DB_query($sql);
        
        // OK, now create new plugin object and insert table header labels
        require_once($_CONF['path_system'] . 'classes/plugin.class.php');
        $plugin_results = new Plugin();
        $plugin_results->plugin_name = $LANG_EVLIST00[1];
        $plugin_results->searchlabel = $LANG_EVLIST00[8];
        $plugin_results->addSearchHeading($LANG_EVLIST00[9]);
        $plugin_results->addSearchHeading($LANG_EVLIST00[10]);
        $plugin_results->addSearchHeading($LANG_EVLIST00[11]);
        $mycount = DB_numRows($result);
        
        // NOTE if any of your data items need to be links then add them here! 
        // make sure data elements are in an array and in the same order as your
        // headings above!
        for ($i = 1; $i <= $mycount; $i++) {
            $A = DB_fetchArray($result);
            if (strlen($A['title']) > 128){
                $title = substr($A['title'], 0, 128) . '...';
            } else {
                $title = $A['title'];
            }
            $title = COM_stripslashes($title);
            $searchcount++;
            $url = $_CONF[site_url]. "/evlist/event.php?eid=" . $A['ID'];
            $row = array("<a href=\"{$url}\">{$title}</a>", COM_stripslashes($A['date_start1']), COM_stripslashes($A['hits']));
            $plugin_results->addSearchResult($row);
        }
        $plugin_results->num_searchresults = $searchcount;
        $plugin_results->num_itemssearched = DB_count($_TABLES['evlist_events']);
        
        return $plugin_results;
    }
}

function plugin_autotags_evlist($op, $content = '', $autotag = '') {
    global $_CONF, $_TABLES;
    
    if ($op == 'tagname') {
        return 'event';
    } elseif ($op == 'parse') {
        $eid = COM_applyFilter($autotag['parm1']);
        $url = COM_buildURL($_CONF['site_url'] . '/evlist/event.php?eid=' . $eid);
        if (empty($autotag['parm2'])) {
            $text = COM_stripslashes(DB_getItem($_TABLES['evlist_events'], 'title', "id = '$eid'"));
        } else {
            $text = $autotag['parm2'];
        }
        $link = '<a href="' . $url . '">' . $text . '</a>';
        $content = str_replace($autotag['tagstr'], $link, $content);
        
        return $content;
    }
}

function plugin_centerblock_evlist($where, $page, $topic = '') {
    global $_CONF, $_USER, $_TABLES, $LANG_EVLIST05;
    
    $result = DB_query("SELECT allow_anon_view, enable_centerblock FROM {$_TABLES['evlist_settings']}");
    list ($allow_anon,$enabled) = DB_fetchArray($result);
    
    if (($allow_anon != '1' && $_USER['uid'] < 2) || $enabled != '1') {
        return;
    }

    // Retrieve Centerblock Settings
    $query = DB_query ("SELECT pos_centerblock,
        topic_centerblock, range_centerblock, limit_centerblock, limit_summary
        FROM {$_TABLES['evlist_settings']}");
    list ($position,$cbtopic,$range,$limit,$length) = DB_fetchArray ($query);
    $cbtopic = COM_stripslashes($cbtopic);
    
    // display on homepage or not at all
    if ($cbtopic == 'home' AND ($page > 1 OR !empty ($topic))) {
        return;
    }
    // display on homepage or not at all
    if (empty($topic) && ($cbtopic != 'home' && $cbtopic != 'all')) {
        return;
    }
    // display on topic page or not at all
    if (!empty($topic) && $cbtopic != $topic && $cbtopic != 'all') {
        return;
    }
    
    // get position from stories query to account for 'after featured' position.
    if ($position == 2) {
        if (($where == 1) && ($_CONF['showfirstasfeatured'] == 0)) {
            if (!empty($topic)) {
                if (DB_count($_TABLES['stories'], array('featured', 'tid', 'draft_flag'), array(1, $topic, 0)) < 1) {
                    // no featured story found in topic - redefine position
                    $position = 1;
                }
            } else {//redefine position if no featured story on front page
                if (DB_count($_TABLES['stories'], array('featured', 'frontpage', 'draft_flag'), array(1, 1, 0)) < 1) {
                    // no featured story found - redefine position
                    $position = 1;
                }
            }
        } elseif ($where == 1) {
            if (!empty($topic)) {//redifine position if no featured story in topic page
                if (DB_count($_TABLES['stories'], array('featured', 'tid', 'draft_flag'), array(1, $topic, 0)) < 1) {
                    // no featured story found - redefine position
                    $position = 1;
                }
            } else {//redifine position if no featured story on front page
                if (DB_count($_TABLES['stories'], array('featured', 'frontpage', 'draft_flag'), array(1, 1, 0)) < 1) {
                    // no featured story found - redefine position
                    $position = 1;
                }
            }
        }
    }
    
    // display in selected position or not at all.
    if ($position != $where) {
        return;
    }
    
    $retval = '';
    
    #
    #if () {
    #    
        $result = evlist_events_query($range, $limit);
        if (empty($result)) {
            return;
        }
        
        $t = new Template($_CONF['path'] . 'plugins/evlist/templates/');
        $t->set_file(array(
            'centerblock' => 'centerblock.thtml',
            'item' => 'centerblock_item.thtml'
        ));
        
        $t->set_var(array(
            'startblock' => COM_startBlock($LANG_EVLIST05[$range]),
            'endblock' => COM_endBlock(),
            'title_label' => $LANG_EVLIST05[5],
            'date_label' => $LANG_EVLIST05[6]
        ));
        
        while ($A = DB_fetchArray($result)) {
            
            if ($A['recurring'] == '1') {
                $timestamp = '&ts=' . strtotime($A['date_start1']);
            } else $timestamp = '';
            
            $titlelink = COM_buildURL($_CONF['site_url'] . '/evlist/event.php?eid=' . $A['id'] . $timestamp);
            $titlelink = '<a href="' . $titlelink . '">' . COM_stripslashes($A['title']) . '</a>';
            
            $summary = strip_tags(COM_stripslashes($A['summary']), '<a>');
            if (!empty($length) && $length >= 1) {
                if (strlen($summary) > $length) {
                    $summary = substr($summary, 0, $length);
                    $summary = $summary . '...';
                }
            }
            
            $startdate = evlist_displayFormatedDate(strtotime($A['date_start1']));
            $cssid = ($cssid ==1) ? 2: 1;
            
            $t->set_var(array(
                'cssid' => $cssid,
                'title' => $titlelink,
                'summary' => $summary,
                'startdate' => $startdate
            ));
            $t->parse('eventrow', 'item', true);
        }
        
        $t->parse('output', 'centerblock');
        $retval .= $t->finish($t->get_var('output'));
    #}
    
    return $retval;
}

function plugin_getListField_evlist($fieldname, $fieldvalue, $A, $icon_arr) {
    global $_CONF, $LANG_ADMIN, $LANG_EVLIST01, $_TABLES;

    switch($fieldname) {
        case "edit":
            $retval = '<a href="' . $_CONF['site_url'] . '/evlist/event.php?op=edit&amp;eid='
                . $A['id'] . '" title="' . $LANG_EVLIST01[49] . '">' . $icon_arr['edit'] . '</a>';
            break;
        case "copy":
            $retval = '<a href="' . $_CONF['site_url'] . '/evlist/event.php?op=clone&amp;eid='
                . $A['id'] . '" title="' . $LANG_EVLIST01[50] . '">' . $icon_arr['copy'] . '</a>';
            break;
        case "title":
            $title = COM_stripslashes ($A['title']);
            $url = $_CONF['site_url'] . '/evlist/event.php?eid=' . $A['id'];
            $retval = '<a href="' . $url . '" title="' . $LANG_EVLIST01[48] . '">' . $title . '</a>';
            break;
        case "status":
            if ($A['status'] == '1') {
                $switch = ' checked="checked"';
            } else {
                $switch = '';
            }
            $form = '<form action="' . $_CONF['site_admin_url'] . '/plugins/evlist/index.php" method="post">'
                . '<input type="checkbox" name="evenable" value="' . $A['id'] . '" onclick="submit()"' . $switch . ' />'
                . '<input type="hidden" name="statusChange" value="' . $A['id'] . '" /></form>';
            $retval = $form;
            break;
        default:
            $retval = $fieldvalue;
            break;
    }
    return $retval;
}

//any events belonging to a user being deleted will
//become the root user's events and they will be disabled.
function plugin_user_delete_evlist($uid) {
    global $_TABLES;
    
    DB_query("UPDATE {$_TABLES['evlist_events']} SET
        owner_id = '2', status = '0'
        WHERE owner_id = '$uid'");
}

function plugin_geticon_evlist() {
    global $_CONF;
    
    return $_CONF['site_url'] . '/evlist/images/evList.png';
}

function plugin_getfeednames_evlist() {
    global $_TABLES;
    
    $feeds = array();
    $feeds[] = array('id' => '0', 'name' => 'all upcoming events');
    
    $result = DB_query("SELECT id, name FROM {$_TABLES['evlist_categories']} WHERE status = 1");
    if (DB_numRows($result) > 0) {
        while ($A = DB_fetchArray($result)) {
            $feeds[] = array('id' => $A['id'], 'name' => $A['name']);
        }
    }
    
    return $feeds;
}

function plugin_getfeedcontent_evlist ($feed, &$link, &$update_data) {
    global $_CONF, $_TABLES, $LANG_EVLIST00;
    
    $content = array();
    $lids = array();
    $ids = array();
    
    if (DB_getItem($_TABLES['evlist_settings'], 'allow_anon_view') != '1') {
        return $content;
    }
    
    $result = DB_query ("SELECT topic,limits,content_length FROM {$_TABLES['syndication']} WHERE fid = $feed");
    $F = DB_fetchArray ($result);
    
    $result = evlist_events_query($range, $F['limits'], $F['topic']);
    
    if (empty($result)) {
        return $content;
    } else {
        while ($A = DB_fetchArray($result)) {
            if ($A['recurring'] == '1') {
                //are we displaying a recurrence?
                //if so add a timestamp to the appropriate urls.
                $timestamp = "&ts=" . strtotime($A['date_start1']);
            } else $timestamp = '';
            
            $link = "{$_CONF['site_url']}/evlist/event.php?eid={$A['id']}" . $timestamp;
            $postmode = $A['postmode'] == '2' ? 'html' : 'plaintext';
            
            if( $A['postmode'] == '1' ) {//plaintext
                $A['summary'] = nl2br($A['summary']);
            }
            
            $ids[] = $A['id'];
            $content[] = array(
                'title'     => COM_stripslashes($A['title']),
                'summary'   => PLG_replaceTags(COM_stripslashes($A['summary'])),
                'link'      => $link,
                'date'      => COM_stripslashes($A['date_start1']),
                'format'    => $postmode
                );
        }
    }
    
    if ($F['topic'] == 0) {
        $link = "{$_CONF['site_url']}/evlist/index.php?op=list";
    } else {
        $link = "{$_CONF['site_url']}/evlist/index.php.php?cat={$F['topic']}";
    }
    
    if (count($ids) > 0 ) {
        $update = implode (',', $ids);
    }
    
    return $content;
}

function plugin_feedupdatecheck_evlist ($feed, $topic, $update_data, $limit) {
    global $_TABLES;
    
    $result = evlist_events_query('', $limit, $topic);
    $num = DB_numRows ($result);

    $ids = array ();
    for ($i = 0; $i < $num; $i++) {
        $A = DB_fetchArray ($result);
        $ids[] = $A['id'];
    }
    $current = implode (',', $ids);
    
    return ($current != $update_data) ? false : true;
}

function plugin_runScheduledTask_evlist() {
    global $_CONF, $_TABLES, $LANG_EVLIST03;
    
    //check if reminders exist
    $result = DB_query("SELECT * FROM {$_TABLES['evlist_remlookup']}");
    
    if (DB_numRows($result) >= 1) {
        
        //common variables
        $from = $_CONF['site_mail']; // Edit this to suit your site's needs.
        $from = COM_formatEmailAddress($_CONF['site_name'], $from);
        $subject = $LANG_EVLIST03[29];
        $msg_top = $LANG_EVLIST03[30];
        $msg_bot = $LANG_EVLIST03[34];
        
        //begin loop
        while ($R = DB_fetchArray($result)) {
            
            //check if event exists
            $Aresult = DB_query("SELECT title, summary FROM {$_TABLES['evlist_events']} WHERE id = '{$R['eid']}'");
            if (DB_numRows($Aresult) < 1) {
                //event no longer exists, lose the reminder
                DB_query("DELETE FROM {$_TABLES['evlist_remlookup']} WHERE id = '{$R['id']}' LIMIT 1");
                continue;
            }
            
            $A = DB_fetchArray($Aresult);
            
            //check if strtotime("now") > date_start + timestamp
            if (!empty($R['timestamp']) && $R['timestamp'] > '0') {
                $date_start = $R['timestamp'];
                $and_ts = '&ts=' . $date_start;
            } else {
                $date_start = $R['date_start'];
                $and_ts = '';
            }
            
            if (strtotime("now") < strtotime("-" . $R['days_notice'] . " days", $date_start)) {
                //continue loop
                continue;
            } else {
                //particular variables
                $title = COM_stripslashes($A['title']);
                $summary = COM_stripslashes($A['summary']);
                $date_start = evlist_displayFormatedDate($date_start);
                $to = COM_stripslashes($R['email']);
                $event_url = $_CONF['site_url'] . '/evlist/event.php?eid=' . $R['eid'] . $and_ts;
                
                $message = $msg_top . LB . LB;
                $message .= sprintf($LANG_EVLIST03[31], $title, $date_start) . LB;
                if (!empty($summary)) {
                    $message .= sprintf($LANG_EVLIST03[32], $summary) . LB . LB;
                }
                $message .= sprintf($LANG_EVLIST03[33], $event_url) . LB . LB;
                $message .= $msg_bot . LB;
                
                //mail reminder
                COM_mail($to, $subject, $message, $from);
                
                //delete reminder
                DB_query("DELETE FROM {$_TABLES['evlist_remlookup']} WHERE id = '{$R['id']}'");
                
                $A['title'] = '';
                $A['summary'] = '';
                $R['timestamp'] = '';
            }
        }
    }
    
    return true;
}

function plugin_submissioncount_evlist() {
    global $_TABLES;
    
    $result = DB_query("SELECT COUNT(*) as count FROM {$_TABLES['evlist_submissions']}");
    list($count) = DB_fetchArray($result);
    
    return $count;
}

function plugin_moderationapprove_evlist($eid) {
    global $_TABLES;
    
    //i don't understand why the whole of the approve stuff cant go here.
    $result = DB_query("UPDATE {$_TABLES['evlist_events']} SET status = '1' WHERE id = '$eid'");
    
    return true;
}

function plugin_moderationdelete_evlist($eid) {
    global $_TABLES;
    
    //DB_delete($_TABLES['evlist_submissions'], 'id', $eid);
    //why does the core have to delete my submission?
    //why shouldn't this function just take care of it?
    DB_delete($_TABLES['evlist_lookup'], 'eid', $eid);
}

function plugin_moderationvalues_evlist() {
    global $_TABLES;
    
    //oy!
    return array(
        'id',
        $_TABLES['evlist_events'],
        "id, title, summary, full_description, date_start1, date_end1, date_start2, date_end2,
        time_start1, time_end1, time_start2, time_end2, url, location, street, city, province,
        country, postal, recurring, rec_option, rec_data, allday, split, contact, email, phone,
        status, postmode, hits, enable_reminders, owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon",
        $_TABLES['evlist_submissions']
    );
}

function plugin_ismoderator_evlist() {
    return SEC_hasRights ('evlist.edit');
}

function plugin_itemlist_evlist() {
    global $_TABLES, $LANG_EVLIST00;
    
    if (plugin_ismoderator_evlist()) {
        $plugin = new Plugin();
        $plugin->submissionlabel = $LANG_EVLIST00[16];
        $plugin->getsubmissionssql = "SELECT id,title,date_start1 as day
                            FROM {$_TABLES['evlist_submissions']}
                            ORDER BY date_start1 ASC";
        $plugin->addsubmissionheading($LANG_EVLIST00[17]);
        $plugin->addsubmissionheading($LANG_EVLIST00[18]);
        
        return $plugin;
    }
}

function plugin_enablestatechange_evlist($enable) {
    global $_TABLES;

    $is_enabled = $enable ? 1 : 0;

    // toggle evlist feeds
    DB_query ("UPDATE {$_TABLES['syndication']} SET is_enabled = $is_enabled WHERE type = 'evlist'");

    // turns block off if on, but will not turn it on if off.  that should be done elsewhere.
    if ($is_enabled != 1) {
        DB_query ("UPDATE {$_TABLES['blocks']} SET is_enabled = $is_enabled WHERE (type = 'phpblock') AND (phpblockfn = 'phpblock_evlist_upcoming')");
    }
}



/*
 * evList, plugin specific, common functions
 */

#function evlist_strtotime($str, $timestamp = '') {
#}

function evlist_sendNotification($A) {
    global $_CONF, $_TABLES, $LANG_EVLIST00;
    
    $title = stripslashes ($A['title']);
    $summary = stripslashes ($A['summary']);
    
    $mailbody = "$LANG_EVLIST00[19]: $title\n"
              . "$LANG_EVLIST00[20]: " . strftime ($_CONF['date'],
                strtotime ($A['date_start1']));
                
    $mailbody .= "\n" . $summary . "\n\n";
    $mailbody .= "$LANG_EVLIST00[21] <{$_CONF['site_admin_url']}/moderation.php>\n\n";
    $mailsubject = $_CONF['site_name'] . ' ' . $LANG_EVLIST00[22];

    $mailbody .= "\n------------------------------\n";
    $mailbody .= "\n$LANG_EVLIST00[23]\n";
    $mailbody .= "\n------------------------------\n";
    COM_mail ($_CONF['site_mail'], $mailsubject, $mailbody);
}

function evlist_hit($eid) {
    global $_TABLES;

    DB_query( "UPDATE {$_TABLES['evlist_events']} SET hits = hits + 1 WHERE ID = '$eid'" );
}

function evlist_events_query($range = '', $limit = '', $category = '') {
    global $_TABLES;
    
    $Sresult = DB_query("SELECT event_passing, limit_list FROM {$_TABLES['evlist_settings']}");
    $S = DB_fetchArray($Sresult);
    
    switch ($S['event_passing']) {
    case 1:
        //event passes when now > start_datetime
        //The seeming complexity of the following where clause is due to the fact
        //that evList does not require a start time.  That's the short version.
        $eventstart = "ADDTIME(CONCAT(date_start1,' ','00:00:00'),time_start1)";
        break;
    case 3:
        //event passes when end_datetime < now
        //problem is that an end date and an end time are not required.
        #$eventstart = "ADDTIME(CONCAT(date_end1,' ','00:00:00'),time_end1)";
        #break;
    case 4:
        //event passes when endate < now
        //problem is that an end date is not required.
        #$eventstart = "date_end1";
        #break;
    case 2:
    default:
        //event passes when now > start_date
        $eventstart = "date_start1";
    }
    
    switch ($range) {
    case 1:// past
        $where = " AND $eventstart < NOW()";
        $asc = " DESC";
        break;
    case 3://this week
        $where = " AND $eventstart BETWEEN NOW() AND NOW() + INTERVAL 7 DAY";
        $asc = " ASC";
        break;
    case 4://this month
        $where = " AND $eventstart BETWEEN NOW() AND NOW() + INTERVAL 1 MONTH";
        $asc = " ASC";
        break;
    #case 5:
        //current -- probably not necessary.
    #    $where = "";
    #    $asc = " ASC";
    #    break;
    case 2://upcoming
    default://upcoming
        $where = " AND $eventstart >= NOW()";
        $asc = " ASC";
    }
    
    if (!empty($category) && $category != '0') {
        $category = COM_applyFilter($category, true);
        $category_leftjoin = " LEFT JOIN {$_TABLES['evlist_lookup']} tl"
            . " ON id = eid";
        $wherecat = " AND (cid = '$category' AND tl.status = '1')";
    }
    
    if (!empty($limit)) {
        if (substr ($limit, -1) == 'h') { // last xx hours
            //this is for the feed update check which could be number of items or hours.
            $hours = substr ($limit, 0, -1);
            $where .= " AND ADDTIME(CONCAT(date_start1,' ','00:00:00'),time_start1) >= DATE_SUB(NOW(),INTERVAL $hours HOUR)";
        } else {
            $limit = ' LIMIT ' . $limit;
        }
    } elseif (!empty($_REQUEST['page'])) {
        //$display_limit = $S['limit_list'];//this is also set in evlist_pagenav()
        $page = COM_applyFilter($_REQUEST['page'], true);
        $limit = ($S['limit_list'] * $page) - $S['limit_list'];
        $limit = " LIMIT $limit, {$S['limit_list']}";
    } else {
        //$display_limit = $S['limit_list'];//this is also set in evlist_pagenav()
        $page = 1;
        $limit = ($S['limit_list'] * $page) - $S['limit_list'];
        $limit = " LIMIT $limit, {$S['limit_list']}";
    }
    
    //get potential recurrences and add them to the result set
    evlist_makeRecurrences();
    
    $result = DB_query("SELECT * FROM evlist_temp_events tt"
        . $category_leftjoin
        . " WHERE tt.status = '1'"
        . $wherecat . $where . COM_getPermSQL('AND')
        . " ORDER BY ADDTIME(CONCAT(date_start1,' ','00:00:00'),time_start1)" . $asc . $limit);
    $nrows = DB_numRows($result);
    if ($nrows < 1) {
        //return empty
        $retval = '';
    } else {
        $retval = $result;
    }
    
    return $retval;
}

function evlist_single_event_query($eid, $access = '2') {
    global $_TABLES;
    
    $result = DB_query("SELECT * FROM {$_TABLES['evlist_events']} WHERE id = '$eid'" . COM_getPermSQL('AND', '0', $access) . " LIMIT 1");
    $nrows = DB_numRows($result);
    if ($nrows < 1) {
        //perhaps this id corresponds to a submission that needs to be edited prior to approval
        $result = DB_query("SELECT * FROM {$_TABLES['evlist_submissions']} WHERE id = '$eid'" . COM_getPermSQL('AND', '0', $access) . " LIMIT 1");
        $nrows = DB_numRows($result);
        if ($nrows < 1 ) {
            //return empty
            $retval = '';
        } else {
            $retval = $result;
        }
    } else {
        $retval = $result;
    }
    
    return $retval;
}

/*
 * creates a tmp table containing all events + new rows for recurrences if any.
 * 
 */
function evlist_makeRecurrences() {
    global $_DB_table_prefix, $_TABLES;
    
    //if the temp table exists, then don't do all this again.
    #$result = DB_query("SHOW TABLES LIKE 'evlist_temp_events'");
    #if( DB_numRows($result) >= 1) {
    #    return;
    #} else {
    DB_query("DROP TABLE IF EXISTS evlist_temp_events");
    
        $result = DB_query("SELECT * FROM {$_TABLES['evlist_events']}
            WHERE status = '1' AND recurring = '1'");// . COM_getPermSQL('AND')); // let the list check perms only
        
        if (DB_numRows($result) < 1) {
            //the list is calling this function and will query a temp table so
            // give it a table to query...
            DB_query("CREATE TEMPORARY TABLE IF NOT EXISTS evlist_temp_events
                (LIKE {$_TABLES['evlist_events']})");
            DB_query("INSERT INTO evlist_temp_events
                (SELECT * FROM {$_TABLES['evlist_events']})");
        } else {
            $valuesA = array();
            $now = strtotime("now");
            
            while ($A = DB_fetchArray($result,false)) {
                $rec_dataA = explode(";", $A['rec_data']);
                //use unix timestamp for calculations on dates.
                $startdate = strtotime($A['date_start1']);
                
                switch ($A['rec_option']) {
                case 1:
                    // daily, date thru date.
                    // return max 62 recurrences if necessary
                    // 1 day = 86400 seconds.
                    // 1 month = 2678400 seconds.
                    $startdate = strtotime("+1 day", $startdate); // so that the initial startdate is not repeated.
                    
                    if (!empty($rec_dataA[0])) {
                        $enddate = strtotime($rec_dataA[0]);
                        if ($enddate < $startdate) {
                            break;
                        }
                        
                        if (($enddate - $startdate) < 2678400) {
                            // if given range is less than 31 days, let is stand
                            $range = ($enddate - $startdate) / 86400;
                        } elseif ($now > $startdate && $now < $enddate) {
                            // if startdate is before now and enddate is after now, but event runs more than 1 month
                            // ensure range is not more than 31 days before or after now
                            if ($now - $startdate > 2678400) {
                                $startdate = strtotime("-1 month", $now);// - 2678400;
                            }
                            if ($endate - $now > 2678400) {
                                $enddate = strtotime("+1 month", $now);// + 2678400;
                            }
                            $range = ($enddate - $startdate) / 86400;
                        } elseif ($now < $startdate) {
                            // startdate is after now and event runs more than 1 month
                            // range will be startdate + 1 month
                            $range = 31;
                        } elseif ($now > $enddate) {
                            // event no longer runs but has run longer than 1 month
                            // range will be 31 days prior to enddate
                            $startdate = strtotime("-1 month", $enddate);// - 2678400;
                            $range = 31;
                        }
                    } else {
                        //need to supply an enddate before we calculate a default range
                        //$range = 31;
                        if ($startdate < strtotime("-1 month", $now)) {
                            //max range 31 days previous and 31 days post now
                            $startdate = strtotime("-1 month", $now);
                            $enddate = strtotime("+1 month", $now);
                            $range = ($enddate - $startdate) / 86400;
                        } elseif ($startdate < $now) {
                            //range is startdate thru now plus 31 days post now
                            $enddate = strtotime("+1 month", $now);//$now + 2678400;
                            $range = ($enddate - $startdate) / 86400;
                        } elseif ($startdate > $now) {
                            //range is 31 days post startdate
                            $enddate = strtotime("+1 month", $startdate);// + 2678400;
                            $range = 31;
                        }
                    }
                    
                    // add 1 day to startdate until last date is reached and insert new rows.
                    for ( $i = 0; $i <= $range; $i++ ) {
                        $valuesAstr = '';
                        
                        //if skip weekends
                        if ($rec_dataA[1] == '1') {
                            if (date("D", $startdate) == 'Sat' || date("D", $startdate) == 'Sun') {
                            // add 1 day to startdate and continue.
                            $startdate = strtotime("+1 day", $startdate);
                                continue;
                            }
                        }
                        
                        foreach ($A as $key => $val) {
                            $val = addslashes($val);
                            if ($key == 'date_start1') {
                                $val = date("Ymd",$startdate);
                            }
                            $valuesAstr .= "'" . $val . "'";
                            if ($key != 'perm_anon') {
                                $valuesAstr .= ",";
                            }
                            $val = '';
                        }
                        
                        $valuesA[] = "($valuesAstr)";
                        // add 1 day to startdate until last date is reached.
                        $startdate = strtotime("+1 day", $startdate);
                    }
                    break;
                
                case 2:
                    // monthly, date thru date
                    // return max 32 recurrences if necessary (= 18 months)
                    // 1 month = 2678400 seconds.
                    // 18 months = 48211200 seconds.
                    $startdate = strtotime("+1 month", $startdate); // so that the initial startdate is not repeated.
                    if (!empty($rec_dataA[0])) {
                        $enddate = strtotime($rec_dataA[0]);
                        if ($enddate < $startdate) {
                            break;
                        }
                        
                        if (($enddate - $startdate) < 48211200) {
                            // if given range is less than 18 months, let is stand
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($now > $startdate && $now < $enddate) {
                            // if startdate is before now and enddate is after now, but event runs more than 18 months
                            // ensure range is not more than 18 months before or after now
                            if ($now - $startdate > 48211200) {
                                $startdate = strtotime("-18 months", $now);//$now - 48211200;
                            }
                            if ($endate - $now > 48211200) {
                                $enddate = strtotime("+18 months", $now);//$now + 48211200;
                            }
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($now < $startdate) {
                            // startdate is after now and event runs more than 18 months
                            // range will be startdate + 18 months
                            $range = 18;
                        } elseif ($now > $enddate) {
                            // event no longer runs but has run longer than 18 months
                            // range will be 18 months prior to enddate
                            $startdate = strtotime("-18 month", $endate);//$enddate - 48211200;
                            $range = 18;
                        }
                    } else {
                        //need to supply an enddate before we calculate a default range
                        //$range = 18;
                        if ($startdate < strtotime("-18 months", $now)) {// - 48211200)) {
                            //max range 18 months previous and 18 months post now
                            $startdate = strtotime("-18 months", $now);
                            $enddate = strtotime("+18 months", $now);
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($startdate < $now) {
                            //range is startdate thru now plus 18 months post now
                            $enddate = strtotime("+18 months", $now);
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($startdate > $now) {
                            //range is 18 months post startdate
                            //$enddate = $startdate + 48211200;
                            $range = 18;
                        }
                    }
                    
                    // add 1 month to startdate until last date is reached and insert new rows.
                    for ( $i = 0; $i <= $range; $i++ ) {
                        $valuesAstr = '';
                        
                        //if skip weekends
                        if (!empty($rec_dataA[1])) {
                            if ($rec_data[1] == '1' && (date("D", $startdate) == 'Sat' || date("D", $startdate) == 'Sun')) {
                                //skip weekends
                                // add 1 month to startdate and continue.
                                $startdate = strtotime("+1 month", $startdate);
                                continue;
                            } elseif ($rec_data[1] == '2') {
                                //reschedule
                                if (date("D", $startdate) == 'Sat') {
                                    $skipnext = 172800;//strtotime("+2 days", $startdate) - $startdate;
                                } elseif (date("D", $startdate) == 'Sun') {
                                    $skipnext = 86400;//strtotime("+1 day", $startdate) - $startdate;
                                } else $skipnext = '0';
                                
                            }
                        }
                        
                        foreach ($A as $key => $val) {
                            $val = addslashes($val);
                            if ($key == 'date_start1') {
                                $val = date("Y-m-d",$startdate + $skipnext);
                            }
                            $valuesAstr .= "'" . $val . "'";
                            if ($key != 'perm_anon') {
                                $valuesAstr .= ",";
                            }
                            $val = '';
                        }
                        
                        $valuesA[] = "($valuesAstr)";
                        // add 1 month to startdate until last date is reached.
                        $startdate = strtotime("+1 month", $startdate);
                    }
                    break;
                
                case 3:
                    // yearly, date thru date
                    // return max 20 recurrences if necessary (= 20 years)
                    // 1 year = 32140800 seconds.
                    // 10 years = 321408000 seconds.
                    $startdate = strtotime("+1 year", $startdate); // so that the initial startdate is not repeated.
                    if (!empty($rec_dataA[0])) {
                        $enddate = strtotime($rec_dataA[0]);
                        //return print (date("Ymd",$enddate) . ',' . date("Ymd",$startdate));
                        if ($enddate < $startdate) {
                            break;
                        }
                        
                        if (($enddate - $startdate) < 321408000) {
                            // if given range is less than 10 years, let is stand
                            $range = ($enddate - $startdate) / 32140800;
                        } elseif ($now > $startdate && $now < $enddate) {
                            // if startdate is before now and enddate is after now, but event runs more than 10 years
                            // ensure range is not more than 10 years before or after now
                            if ($now - $startdate > 321408000) {
                                $startdate = strtotime("-10 years", $now);
                            }
                            if ($endate - $now > 321408000) {
                                $enddate = strtotime("+10 years", $now);
                            }
                            $range = ($enddate - $startdate) / 32140800;
                        } elseif ($now < $startdate) {
                            // startdate is after now and event runs more than 10 years
                            // range will be startdate + 10 years
                            $range = 10;
                        } elseif ($now > $enddate) {
                            // event no longer runs but has run longer than 10 years
                            // range will be 10 years prior to enddate
                            $startdate = strtotime("-10 years", $endate);
                            $range = 10;
                        }
                    } else {
                        //need to supply an enddate before we calculate a default range
                        //$range = 10;
                        if ($startdate < strtotime("-10 years", $now)) {
                            //max range 10 years previous and 10 years post now
                            $startdate = strtotime("-10 years", $now);
                            $enddate = strtotime("+10 years", $now);
                            $range = ($enddate - $startdate) / 32140800;
                        } elseif ($startdate < $now) {
                            //range is startdate thru now plus 10 years post now
                            $enddate = strtotime("+10 years", $now);
                            $range = ($enddate - $startdate) / 32140800;
                        } elseif ($startdate > $now) {
                            //range is 18 months post startdate
                            //$enddate = $startdate + 48211200;
                            $range = 10;
                        }
                    }
                    
                    // add 1 year to startdate until last date is reached and insert new rows.
                    for ( $i = 0; $i <= $range; $i++ ) {
                        $valuesAstr = '';
                        
                        //if skip weekends
                        if (!empty($rec_dataA[1])) {
                            if ($rec_data[1] == '1' && (date("D", $startdate) == 'Sat' || date("D", $startdate) == 'Sun')) {
                                //skip weekends
                                // add 1 year to startdate and continue
                                $startdate = strtotime("+1 year", $startdate);
                                continue;
                            } elseif ($rec_data[1] == '2') {
                                //reschedule
                                if (date("D", $startdate) == 'Sat') {
                                    $skipnext = 172800;//strtotime("+2 days", $startdate) - $startdate;
                                } elseif (date("D", $startdate) == 'Sun') {
                                    $skipnext = 86400;//strtotime("+1 day", $startdate) - $startdate;
                                } else $skipnext = '0';
                                
                            }
                        }
                        
                        foreach ($A as $key => $val) {
                            $val = addslashes($val);
                            if ($key == 'date_start1') {//changing startdate for new row
                                $val = date("Y-m-d",$startdate + $skipnext);
                            }
                            $valuesAstr .= "'" . $val . "'";
                            if ($key != 'perm_anon') {//this should be the last field in the row
                                $valuesAstr .= ",";
                            }
                            $val = '';
                        }
                        
                        $valuesA[] = "($valuesAstr)";
                        // add 1 year to startdate until last date is reached.
                        $startdate = strtotime("+1 year", $startdate);
                    }
                    break;
                
                case 4:
                    // weekly by day, e.g., mon, wed, fri
                    if (empty($rec_dataA[0])) {
                        break;
                    }
                    $startdate = strtotime("+1 day", $startdate);// so that the initial startdate is not repeated.
                    //$weekbegins: 1 = sunday; 2 = monday
                    $weekbegins = DB_getItem($_TABLES['evlist_settings'], 'week_begins');
                    if ($weekbegins == '1') {
                        $nextdayA = array(1 => 'Sunday', 2 => 'Monday', 3 => 'Tuesday',
                            4 => 'Wednesday', 5 => 'Thursday', 6 => 'Friday', 7 => 'Saturday');
                    } else {
                        $nextdayA = array(1 => 'Monday', 2 => 'Tuesday', 3 => 'Wednesday',
                            4 => 'Thursday', 5 => 'Friday', 6 => 'Saturday', 7 => 'Sunday');
                    }
                    
                    //$daynum is the array of weekdays indicated by the number of the day.
                    $daynumA = explode(",", $rec_dataA[0]);
                    
                    if (!empty($rec_dataA[1])) {
                        $enddate = strtotime($rec_dataA[1]);
                        if ($enddate < $startdate) {
                            break;
                        }
                        
                        // 1 week = 604800 seconds
                        if (($enddate - $startdate) < '') {
                            // if given range is less than 16 weeks, let it stand
                            $range = ($enddate - $startdate) / 604800;
                        } elseif ($now > $startdate && $now < $enddate) {
                            // if startdate is before now and enddate is after now, but event runs more than 16 weeks
                            // ensure range is not more than 16 weeks before or after now
                            if ($now - $startdate > 9676800) {
                                $startdate = strtotime("-16 weeks", $now);
                            }
                            if ($endate - $now > 9676800) {
                                $enddate = strtotime("+16 weeks", $now);
                            }
                            $range = ($enddate - $startdate) / 604800;
                        } elseif ($now < $startdate) {
                            // startdate is after now and event runs more than 16 weeks
                            // range will be startdate + 16 weeks
                            $range = 16;
                        } elseif ($now > $enddate) {
                            // event no longer runs but has run longer than 16 weeks
                            // range will be 16 weeks prior to enddate
                            $startdate = strtotime("-16 weeks", $enddate);
                            $range = 16;
                        }
                    } else {
                        //need to supply an enddate before we calculate a default range
                        //$range = 16
                        if ($startdate < strtotime("-16 weeks", $now)) {
                            //max range 16 weeks previous and 16 weeks post now
                            $startdate = strtotime("-16 weeks", $now);
                            $enddate = strtotime("+16 weeks", $now);
                            $range = ($enddate - $startdate) / 604800;
                        } elseif ($startdate < $now) {
                            //range is startdate thru now plus 16 weeks post now
                            $enddate = strtotime("+16 weeks", $now);
                            $range = ($enddate - $startdate) / 604800;
                        } elseif ($startdate > $now) {
                            //range is 16 weeks post startdate
                            $enddate = strtotime("+16 weeks", $startdate);
                            $range = 16;
                        }
                    }
                    
                    // add 1 week to startdate until last date is reached and insert new rows.
                    for ( $i = 0; $i <= $range; $i++ ) {//loop thru the range
                        
                        foreach ($daynumA as $daykey => $daynum) {//loop thru the week
                            $valuesAstr = '';
                            $nextday = strtotime("next " . $nextdayA[$daynum], $startdate);
                            
                            //and incase the enddate is in the middle of the week do a check
                            if ($enddate < $nextday) {
                                break 2;
                            }
                            
                            foreach ($A as $key => $val) {//loop thru the event row
                                $val = addslashes($val);
                                if ($key == 'date_start1') {
                                    $val = date("Y-m-d", $nextday);
                                }
                                $valuesAstr .= "'" . $val . "'";
                                if ($key != 'perm_anon') {
                                    $valuesAstr .= ",";
                                }
                                $val = '';
                            }
                            $valuesA[] = "($valuesAstr)";
                        }
                        
                        // add 1 week to startdate until last date is reached.
                        $startdate = strtotime("+1 week", $startdate);
                    }
                    break;
                case 5:
                    //according to a particular interval per month
                    if (empty($rec_dataA[0]) || empty($rec_dataA[1])) {
                        break;
                    }
                    // return max 32 recurrences if necessary (= 18 months)
                    // 1 month = 2678400 seconds.
                    // 18 months = 48211200 seconds.
                    
                    if (!empty($rec_dataA[2])) {
                        //$enddate = strtotime($rec_dataA[2]);
                        //in case the last day of the month is the last day we're looking for
                        //we wouldn't want to miss it so add 2 days just to get beyond it.
                        $enddate = strtotime("+2 days", strtotime($rec_dataA[2]));
                        if ($enddate < $startdate) {
                            break;
                        }
                        
                        if (($enddate - $startdate) < 48211200) {
                            // if given range is less than 18 months, let is stand
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($now > $startdate && $now < $enddate) {
                            // if startdate is before now and enddate is after now, but event runs more than 18 months
                            // ensure range is not more than 18 months before or after now
                            if ($now - $startdate > 48211200) {
                                $startdate = strtotime("-18 months", $now);
                            }
                            if ($endate - $now > 48211200) {
                                $enddate = strtotime("+18 months", $now);
                            }
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($now < $startdate) {
                            // startdate is after now and event runs more than 18 months
                            // range will be startdate + 18 months
                            $range = 18;
                        } elseif ($now > $enddate) {
                            // event no longer runs but has run longer than 18 months
                            // range will be 18 months prior to enddate
                            $startdate = strtotime("-18 month", $endate);
                            $range = 18;
                        }
                    } else {
                        //need to supply an enddate before we calculate a default range
                        //$range = 18;
                        if ($startdate < strtotime("-18 months", $now)) {
                            //max range 18 months previous and 18 months post now
                            $startdate = strtotime("-18 months", $now);
                            $enddate = strtotime("+18 months", $now);
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($startdate < $now) {
                            //range is startdate thru now plus 18 months post now
                            $enddate = strtotime("+18 months", $now);
                            $range = ($enddate - $startdate) / 2678400;
                        } elseif ($startdate > $now) {
                            //range is 18 months post startdate
                            //$enddate = $startdate + 48211200;
                            $range = 18;
                        }
                    }
                    
                    // add 1 month to startdate until last date is reached and insert new rows.
                    for ( $i = 0; $i <= $range; $i++ ) {
                        $valuesAstr = '';
                        
                        // first day of month relative to $startdate
                        $firstday = strtotime(date("Y", $startdate) . date("m", $startdate) . "01");
                        // last day of month relative to $startdate
                        $lastday = strtotime("-1 second", strtotime("+1 month", strtotime(date("Y", $startdate) . date("m", $startdate) . "01")));
                        
                        $weekdayA = array(1 => 'Sunday', 2 => 'Monday', 3 => 'Tuesday',
                            4 => 'Wednesday', 5 => 'Thursday', 6 => 'Friday', 7 => 'Saturday');
                        
                        if ($rec_dataA[0] == 5) {
                            //check if last day of month is day we're looking for before week go back.
                            if (date("l",$lastday) == $weekdayA[$rec_dataA[1]]) {
                                $newdate = $lastday;
                            } else {
                                $newdate = strtotime("last " . $weekdayA[$rec_dataA[1]], $lastday);
                            }
                        } else {
                            $newdate = strtotime("next " . $weekdayA[$rec_dataA[1]], strtotime("+" . $rec_dataA[0] - 1 . " weeks", $firstday));
                            //$newdate = strtotime("+" . $rec_data[0]-1 . " weeks " . $weekdayA[$rec_dataA[1]], $firstday);
                        }
                        
                        //check beginning and end of range lest we add/miss a day
                        #if ($startdate > $newdate) {
                        #    $startdate = strtotime("+1 week", $startdate);
                        #    continue;
                        #} else
                        if ($enddate < $newdate) {
                            break;
                        }
                        
                        foreach ($A as $key => $val) {
                            $val = addslashes($val);
                            if ($key == 'date_start1') {
                                $val = date("Y-m-d",$newdate);
                            }
                            $valuesAstr .= "'" . $val . "'";
                            if ($key != 'perm_anon') {
                                $valuesAstr .= ",";
                            }
                            $val = '';
                        }
                        
                        $valuesA[] = "($valuesAstr)";
                        // add 1 month to startdate until last date is reached.
                        $startdate = strtotime("+1 month", $startdate);
                    }
                    break;
                case 6:
                    //custom dates
                    if (empty($rec_dataA[0])) {
                        break;
                    }
                    $listdatesA = explode(",", $rec_dataA[0]);
                    $countdates = count($listdatesA);
                    
                    //insert new row per startdate in array
                    for ($i = 0; $i < $countdates; $i++) {
                        $valuesAstr = '';
                        $startdate = strtotime($listdatesA[$i]);//don't need this
                        // and oh, btw, there's very little validation going on if you haven't noticed.
                        
                        foreach ($A as $key => $val) {
                            $val = addslashes($val);
                            if ($key == 'date_start1') {//changing startdate for new row
                                $val = date("Y-m-d", $startdate);
                            }
                            $valuesAstr .= "'" . $val . "'";
                            if ($key != 'perm_anon') {//this should be the last field in the row
                                $valuesAstr .= ",";
                            }
                            $val = '';
                        }
                        
                        $valuesA[] = "($valuesAstr)";
                    }
                    break;
                }
            }
            
            $numv = count($valuesA);
            $valuesSTR = '';
            $i = 1;
            
            foreach ($valuesA as $val) {
                $valuesSTR .= $val;
                if ($i < $numv) {
                    $valuesSTR .= ',';
                }
                $i++;
            }
            
            #if (empty($valuesSTR)) {
            #    $valuesSTR = "()";
            #}
            
            #create tmp table; insert rows.
            #leave the range query up to the list function because
            #the tmp table should not yet be dropped.
            
            DB_query("CREATE TEMPORARY TABLE evlist_temp_events
                (LIKE {$_TABLES['evlist_events']})");
            DB_query("INSERT INTO evlist_temp_events
                (SELECT * FROM {$_TABLES['evlist_events']})");
            if (!empty($valuesSTR)) {
                //DB_query("ALTER TABLE evlist_temp_events CHANGE id id VARCHAR(40) NOT NULL");
                DB_query("ALTER TABLE evlist_temp_events DROP PRIMARY KEY");
                DB_query("INSERT INTO evlist_temp_events
                    VALUES " . $valuesSTR);
            }
        }
    #}
}

function evlist_pagenav($range = '2', $category = '') {
    global $_CONF, $_TABLES, $LANG_EVLIST02;
    
    $retval = '';
    $Sresult = DB_query("SELECT event_passing, limit_list FROM {$_TABLES['evlist_settings']}");
    $S = DB_fetchArray($Sresult);
    
    switch ($S['event_passing']) {
    case 1:
        //The seeming complexity of the following where clause is due to the fact
        //that evList does not require a start time.  That's the short version.
        $eventstart = " ADDTIME(CONCAT(date_start1,' ','00:00:00'),time_start1)";
        break;
    case 3:
        //
        break;
    case 4:
        //
        break;
    case 2:
    default:
        $eventstart = " date_start1";
    }
        
    if (!empty($_REQUEST['page'])) {
        $page = COM_applyFilter($_REQUEST['page'], true);
    } else {
        $page = 1;
    }
    
    switch ($range) {
    case 1:// past
        $where = " AND $eventstart < NOW()";
        $range = 'range=1';
        break;
    case 3://this week
        $where = " AND $eventstart BETWEEN NOW() AND NOW() + INTERVAL 7 DAY";
        $range = 'range=3';
        break;
    case 4://this month
        $where = " AND $eventstart BETWEEN NOW() AND NOW() + INTERVAL 1 MONTH";
        $range = 'range=4';
        break;
    case 2://upcoming
    default://upcoming
        $where = " AND $eventstart >= NOW()";
        $range = 'range=2';
    }
    
    if (!empty($category)) {
        $category = COM_applyFilter($category, true);
        $category_leftjoin = " LEFT JOIN {$_TABLES['evlist_lookup']} tl"
            . " ON id = eid";
        $wherecat = " AND (cid = '$category' AND tl.status = '1')";
        $andcat = '&cat=' . $category;
    }

    $result = DB_query("SELECT COUNT(*) AS numrows FROM evlist_temp_events tt"
        . $category_leftjoin
        . " WHERE tt.status='1'"
        . $wherecat . $where . COM_getPermSQL('AND'));
    list($numrows) = DB_fetchArray($result);
    
    if ($numrows > $S['limit_list']) {
        $numpages = ceil($numrows/$S['limit_list']);
        $baseurl = $_CONF['site_url'] . '/evlist/index.php?' . $range . $andcat;
        
        $retval = COM_printPageNavigation ($baseurl, $page, $numpages);
    }
    
    return $retval;
}

function evlist_check_perms($eid, $access = '2') {
    global $_TABLES, $_USER;
    
    if ($access == '3' && !SEC_hasRights('evlist.edit')) {
        if (DB_getItem($_TABLES['evlist_settings'], 'allow_user_add') != '1' && $_USER['uid'] > 1) {
            return FALSE;
        }
    }
    
    $result = evlist_single_event_query($eid, $access);
    if (empty($result)) {
        return FALSE;
    } else {
        return TRUE;
    }
}

function evlist_check_required_fields($A, $form = 'edit') {
    $editFormReqFields = array('title','startyear1','startmonth1');
    
    if ($form == 'edit') {
        $i = 0;
        foreach ($editFormReqFields as $field) {
            if (empty ($A[$field])) {
                $i++;
            }
        }
    }
    if ($i >= 1) {
        return false;
    } else {
        return true;
    }
}

function evlist_checkdates_simple($A) {
    //start month and year are required fields so should never be empty
    $A['startday1'] = COM_applyFilter($A['startday1'],true);
    
    if (empty($A['startday1'])) {
        $A['startday1'] = '01';
    }
    
    $A['date_start1'] = COM_applyFilter($A['startyear1'],true) . '-' . COM_applyFilter($A['startmonth1'],true) . '-' . $A['startday1'];
    
    $A['starthour1'] = COM_applyFilter($A['starthour1'],true);
    $A['startminute1'] = COM_applyFilter($A['startminute1'],true);
    
    if(!empty($A['starthour1'])) {
        if (empty($A['startminute1'])) {
            $A['startminute1'] = '00';
        }
        $A['time_start1'] = $A['starthour1'] . ':' . $A['startminute1'] . ':00';
    }
    
    $A['endday1'] = COM_applyFilter($A['endday1'],true);
    $A['endmonth1'] = COM_applyFilter($A['endmonth1'],true);
    $A['endyear1'] = COM_applyFilter($A['endyear1'],true);
    
    if (!empty($A['endyear1'])) {
        if (!empty($A['endmonth1'])) {
            if (empty($A['endday1'])) {
                $A['endday1'] = '01';
            }
        } else {
            $A['endmonth1'] = '01';
            $A['endday1'] = '01';
        }
        $A['date_end1'] = $A['endyear1'] . '-' . $A['endmonth1'] . '-' . $A['endday1'];
        
        $A['endhour1'] = COM_applyFilter($A['endhour1'],true);
        $A['endminute1'] = COM_applyFilter($A['endminute1'],true);
        
        if (!empty($A['endhour1'])) {
            if (empty($A['endminute1'])) {
                $A['endminute1'] = '00';
            }
            $A['time_end1'] = $A['endhour1'] . ':' . $A['endminute1'] . ':00';
        }
        
        //ensure that end date doesn't occur prior to start date
        $datestart1 = $A['date_start1'] . ' ' . $A['time_start1'];
        $dateend1 = $A['date_end1'] . ' ' . $A['time_end1'];
        
        if (strtotime($dateend1) < strtotime($datestart1)) {
            return false;
        }
    }
    
    if ($A['split'] == '1') {
        
        $A['startday2'] = COM_applyFilter($A['startday2'],true);
        $A['startmonth2'] = COM_applyFilter($A['startmonth2'],true);
        $A['startyear2'] = COM_applyFilter($A['startyear2'],true);
        
        if(!empty($A['startyear2'])) {
            if (!empty($A['startmonth2'])) {
                if (empty($A['startday2'])) {
                    $A['startday2'] = '01';
                }
            } else {
                $A['startmonth2'] = '01';
                $A['startday2'] = '01';
            }
            $A['date_start2'] = $A['startyear2'] . '-' . $A['startmonth2'] . '-' . $A['startday2'];
            
            $A['starthour2'] = COM_applyFilter($A['starthour2'],true);
            $A['startminute2'] = COM_applyFilter($A['startminute2'],true);
            
            if(!empty($A['starthour2'])) {
                if (empty($A['startminute2'])) {
                    $A['startminute2'] = '00';
                }
                $A['time_start2'] = $A['starthour2'] . ':' . $A['startminute2'] . ':00';
            }
            
            $A['endday2'] = COM_applyFilter($A['endday2'],true);
            $A['endmonth2'] = COM_applyFilter($A['endmonth2'],true);
            $A['endyear2'] = COM_applyFilter($A['endyear2'],true);
            
            if (!empty($A['endyear2'])) {
                if (!empty($A['endmonth2'])) {
                    if (empty($A['endday2'])) {
                        $A['endday2'] = '01';
                    }
                } else {
                    $A['endmonth2'] = '01';
                    $A['endday2'] = '01';
                }
                $A['date_end2'] = $A['endyear2'] . '-' . $A['endmonth2'] . '-' . $A['endday2'];
                
                $A['endhour2'] = COM_applyFilter($A['endhour2'],true);
                $A['endminute2'] = COM_applyFilter($A['endminute2'],true);
                
                if (!empty($A['endhour2'])) {
                    if (empty($A['endminute2'])) {
                        $A['endminute2'] = '00';
                    }
                    $A['time_end2'] = $A['endhour2'] . ':' . $A['endminute2'] . ':00';
                }
                
                //ensure that end date doesn't occur prior to start date
                $datestart = $A['date_start2'] . ' ' . $A['time_start2'];
                $dateend = $A['date_end2'] . ' ' . $A['time_end2'];
                if (strtotime($dateend) < strtotime($datestart)) {
                    return false;
                }
            }
        }
    }
    
    return true;
}

function evlist_getRecurringFormatOptions($selected = '') {
    global $LANG_EVLIST06;
    
    $format_options = '';
    $numformats = count($LANG_EVLIST06);
    
    for ($i = 1; $i <= $numformats; $i++) {
        $val = $i;
        $format_options .= '<option value="' . $val . '"';
        if ($i == $selected) {
            $format_options .= ' selected="selected"';
        }
        $format_options .= '>' . $LANG_EVLIST06[$val] . '</option>';
    }
    
    return $format_options;
}

function evlist_getRecurringWeekdayOptions($selected = '1') {
    global $LANG_EVLIST10;
    
    $weekday_options = '';
    $numweekdays = count($LANG_EVLIST10);
    
    for ($i = 1; $i <= $numweekdays; $i++) {
        $val = $i;
        $weekday_options .= '<option value="' . $val . '"';
        if ($i == $selected) {
            $weekday_options .= ' selected="selected"';
        }
        $weekday_options .= '>' . $LANG_EVLIST10[$val] . '</option>';
    }
    
    return $weekday_options;
}

function evlist_getRecurringIntervalOptions($selected = '1') {
    global $LANG_EVLIST11;
    
    $interval_options = '';
    $numintervals = count($LANG_EVLIST11);
    
    for ($i = 1; $i <= $numintervals; $i++) {
        $val = $i;
        $interval_options .= '<option value="' . $val . '"';
        if ($i == $selected) {
            $interval_options .= ' selected="selected"';
        }
        $interval_options .= '>' . $LANG_EVLIST11[$val] . '</option>';
    }
    
    return $interval_options;
}

function evlist_getEventPassingOptions($selected = '') {
    global $LANG_EVLIST13;
    
    $passing_options = '';
    $numoptions = count($LANG_EVLIST13);
    
    for ($i = 1; $i <= $numoptions; $i++) {
        $val = $i;
        $passing_options .= '<option value="' . $val . '"';
        if ($i == $selected) {
            $passing_options .= ' selected="selected"';
        }
        $passing_options .= '>' . $LANG_EVLIST13[$val] . '</option>';
    }
    
    return $passing_options;
}

function evlist_getPostmodeOptions($selected = '') {
    global $LANG_EVLIST08;
    
    $postmode_options = '';
    $numpostmodes = count($LANG_EVLIST08);
    
    for ($i = 1; $i <= $numpostmodes; $i++) {
        $val = $i;
        $postmode_options .= '<option value="' . $val . '"';
        if ($val == $selected) {
            $postmode_options .= ' selected="selected"';
        }
        $postmode_options .= '>' . $LANG_EVLIST08[$val] . '</option>';
    }
    
    return $postmode_options;
}

function evlist_getRangeOptions($selected = '') {
    global $LANG_EVLIST09;
    
    $range_options = '';
    $numranges = count($LANG_EVLIST09);
    
    for ($i = 1; $i <= $numranges; $i++) {
        $val = $i;
        $range_options .= '<option value="' . $val . '"';
        if ($val == $selected) {
            $range_options .= ' selected="selected"';
        }
        $range_options .= '>' . $LANG_EVLIST09[$val] . '</option>';
    }
    
    return $range_options;
}

function evlist_getWeekStartOptions($selected = '1') {
    global $LANG_EVLIST01;
    
    $ws_options = '';
    
    $ws_options .= '<option value="1"';
    if ($selected == '1') {
        $ws_options .= ' selected="selected"';
    }
    $ws_options .= '>' . $LANG_EVLIST01[25] . '</option>';
    $ws_options .= '<option value="2"';
    if ($selected == '2') {
        $ws_options .= ' selected="selected"';
    }
    $ws_options .= '>' . $LANG_EVLIST01[26] . '</option>';
    
    return $ws_options;
}

function evlist_getUsermenuOptions($selected = '1') {
    global $LANG_EVLIST01;
    
    $um_options = '';
    
    $um_options .= '<option value="1"';
    if ($selected == '1') {
        $um_options .= ' selected="selected"';
    }
    $um_options .= '>' . $LANG_EVLIST01[23] . '</option>';
    $um_options .= '<option value="2"';
    if ($selected == '2') {
        $um_options .= ' selected="selected"';
    }
    $um_options .= '>' . $LANG_EVLIST01[24] . '</option>';
    
    return $um_options;
}

function evlist_getTopicOptions($selected = 'home') {
    global $_TABLES, $LANG_EVLIST01;
    
    $options = '<option value="all"';
    if ($selected == 'all') {
        $options .= ' selected="selected"';
    }
    $options .= '>' . $LANG_EVLIST01[28] . '</option>';
    
    $options .= '<option value="home"';
    if ($selected == 'home') {
        $options .= ' selected="selected"';
    }
    $options .= '>' . $LANG_EVLIST01[29] . '</option>';
    
    $result = DB_query("SELECT tid, topic FROM {$_TABLES['topics']}" . COM_getPermSQL('WHERE'));
    if (DB_numRows($result) >= 1) {
        while ($A = DB_fetchArray($result)) {
            $tid = COM_stripslashes($A['tid']);
            $topic = COM_stripslashes($A['topic']);
            $options .= '<option value="' . $tid . '"';
            if ($selected == $tid) {
                $options .= ' selected="selected"';
            }
            $options .= '>' . $topic . '</option>';
        }
    }
    
    return $options;
}

function evlist_getPositionOptions($selected = '1') {
    global $LANG_EVLIST01;
    
        $options .= '<option value="1"';
        if ($selected == '1') {//top of page
            $options .= ' selected="selected"';
        }
        $options .= '>' . $LANG_EVLIST01[30] . '</option>';
        $options .= '<option value="2"';
        if ($selected == '2') {//after featured story
            $options .= ' selected="selected"';
        }
        $options .= '>' . $LANG_EVLIST01[31] . '</option>';
        $options .= '<option value="3"';
        if ($selected == '3') {//bottom of page
            $options .= ' selected="selected"';
        }
        $options .= '>' . $LANG_EVLIST01[32] . '</option>';
        $options .= '<option value="0"';
        if ($selected == '0') {//entire page
            $options .= ' selected="selected"';
        }
        $options .= '>' . $LANG_EVLIST01[33] . '</option>';
        
    return $options;
}

function evlist_getDateFormatOptions($selected = '1') {
    global $_TABLES;
    
    $options = '';
    $result = DB_query("SELECT * FROM {$_TABLES['evlist_dateformat']}");
    while ($A = DB_fetchArray($result)) {
        $options .= '<option value="' . $A['id'] . '"';
        if ($selected == $A['id']) {
            $options .= ' selected="selected"';
        }
        $options .= '>' . strftime($A['format']) . '</option>';
    }
    
    return $options;
}

function evlist_getTimeFormatOptions($selected) {
    global $_TABLES;
    
    $options = '';
    $result = DB_query("SELECT * FROM {$_TABLES['evlist_timeformat']}");
    while ($A = DB_fetchArray($result)) {
        $options .= '<option value="' . $A['id'] . '"';
        if ($selected == $A['id']) {
            $options .= ' selected="selected"';
        }
        $options .= '>' . strftime($A['format']) . '</option>';
    }
    
    return $options;
}

function evlist_getCategoryOptions($selected) {
    global $_TABLES;
    
    $options = '';
    $result = DB_query("SELECT id, name FROM {$_TABLES['evlist_categories']} WHERE status = '1'");
    while ($A = DB_fetchArray($result)) {
        $options .= '<option value="' . $A['id'] . '"';
        if ($selected == $A['id']) {
            $options .= ' selected="selected"';
        }
        $options .= '>' . COM_stripslashes($A['name']) . '</option>';
    }
    
    return $options;
}

//returns a formatted date given a timestamp.
function evlist_displayFormatedDate($timestamp) {
    global $_TABLES;
    
    $result = DB_query("SELECT format FROM {$_TABLES['evlist_dateformat']} WHERE id = (SELECT date_format FROM {$_TABLES['evlist_settings']})");
    list ($format) = DB_fetchArray($result);
    if (empty($timestamp)) {
        $timestamp = time();
    }
    if (!is_numeric($timestamp)) {
        $retval = strftime($format, strtotime($timestamp));
    } else {
        $retval = strftime($format, $timestamp);
    }
    
    return $retval;
}

function evlist_displayFormatedTime($timestamp) {
    global $_TABLES;
    
    $result = DB_query("SELECT format FROM {$_TABLES['evlist_timeformat']} WHERE id = (SELECT time_format FROM {$_TABLES['evlist_settings']})");
    list ($format) = DB_fetchArray($result);
    if (empty($timestamp)) {
        $timestamp = time();
    }
    if (!is_numeric($timestamp)) {
        $retval = strftime($format, strtotime($timestamp));
    } else {
        $retval = strftime($format, $timestamp);
    }
    
    return $retval;
}

//returns the last day of a given/current year/month
function evlist_getLastDay($year = '',$month = '') {
    if (empty($year)) {
        $year = date("Y");
    }
    if (empty($month)) {
        $month = date("m");
    }
    $lastday = 01;
    while (checkdate($month,$lastday,$year)) {
        $lastday++;
    }
    --$lastday;
    
    return $lastday;
}

function evlist_display_email($str) {
    $retval = '';
    for ($i = 0; $i < strlen($str); $i++) {
        $retval .= '&#x' . bin2hex(substr($str, $i, 1)) . ';';
    }
    return $retval;
}

function evlist_display_adminheader($page){
    global $_CONF, $LANG_ADMIN, $LANG_EVLIST01;
    
    
    $t = new Template($_CONF['path'] . 'plugins/evlist/templates/');
    $t->set_file('header','admin_header.thtml');
    
    $evlistadmin = '<a href="' . $_CONF['site_admin_url']
        . '/plugins/evlist/index.php">' . $LANG_EVLIST01[52] . '</a>';
    $adminlink = '<a href="' . $_CONF['site_admin_url']
        . '">' . $LANG_ADMIN['admin_home'] . '</a>';
        
    $t->set_var(array(
        'evl_admin_link' => $evlistadmin . ' | ',
        'admin_link' => $adminlink,
        'imagesrc' => plugin_geticon_evlist()
    ));
    
    if ($page == 'config'){
        $categlink = '<a href="' . $_CONF['site_admin_url']
            . '/plugins/evlist/category.php">' . $LANG_EVLIST01[51] . '</a>';
        $t->set_var('cat_conf_link', $categlink . ' | ');
        $t->set_var('instructions', $LANG_EVLIST01[59]);
    } elseif ($page == 'categ'){
        $configlink = '<a href="' . $_CONF['site_admin_url']
            . '/plugins/evlist/index.php?mode=config">' . $LANG_EVLIST01[44] . '</a>';
        $t->set_var('cat_conf_link', $configlink . ' | ');
        $t->set_var('instructions', $LANG_EVLIST01[58]);
    }
    
    $t->parse ('output','header');
    $retval .= $t->finish($t->get_var('output'));
    
    return $retval;
}

//provides an adjustable list of events in a block
function phpblock_evlist_upcoming() {
    global $_CONF, $_TABLES, $LANG_EVLIST05;
    
    //range to be set from admin panels sometime later, or not.
    // A better idea would be to copy the block, change the name, and edit the range.
    // then you have multiple event blocks, each displaying a different range.
    // Enable only the block you want to display.
    
    //1=past,2=upcoming,3=thisweek,4=thismonth see $LANG_EVLIST09.
    $range = '2';//the block title reflecting this value is set initially when the plugin is installed.
    
    $limit = DB_getItem($_TABLES['evlist_settings'], 'limit_block');// 5 is default
    
    //length to be set from admin panels sometime later, or not.
    $length = '96';
    
    $result = evlist_events_query($range, $limit);
    if (empty($result)) {
        //return empty list msg
        $retval = '';
    } else {
        $t = new Template($_CONF['path'] . 'plugins/evlist/templates/');
        $t->set_file('block','phpblock.thtml');
        $listlink = COM_buildURL($_CONF['site_url'] . '/evlist/index.php?op=list');
        $listlink = '<a href="' . $listlink . '">' . $LANG_EVLIST05[7] . '</a>';
        $t->set_var('listlink', $listlink);
        //populate list
        while ($A = DB_fetchArray($result)) {
            if ($A['recurring'] == '1') {
                $timestamp = '&ts=' . strtotime($A['date_start1']);
            } else $timestamp = '';
            
            $titlelink = COM_buildURL($_CONF['site_url'] . '/evlist/event.php?eid=' . $A['id'] . $timestamp);
            $titlelink = '<a href="' . $titlelink . '">' . COM_stripslashes($A['title']) . '</a>';
            //displaying summaries in the block will be set from the admin panels sometime later.
            $summary = COM_stripslashes($A['summary']);
            if (strlen($summary) > $length) {
                $summary = substr($summary, 0, $length);
                $summary = $summary . '...';
            }
            $startdate = evlist_displayFormatedDate(strtotime($A['date_start1']));
            $t->set_file('block_item','block_item.thtml');
            $t->set_var(array(
                'title' => $titlelink,
                'start_date' => $startdate,
                'summary' => $summary
                ));
            $t->parse ('item','block_item', true);
        }
        $t->parse ('output','block');
        $retval .= $t->finish($t->get_var('output'));
    }
    
    return $retval;
}

?>